==> unicore-gateway-6.4.0-2.src.tar.gz: src/main/java/eu/unicore/gateway/client/AuthSSLProtocolSocketFactory.java <==
package eu.unicore.gateway.client;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.SocketAddress;
import java.net.UnknownHostException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import org.apache.log4j.Logger;

import javax.net.SocketFactory;
import javax.net.ssl.KeyManager;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.TrustManagerFactory;

import org.apache.commons.httpclient.ConnectTimeoutException;
import org.apache.commons.httpclient.params.HttpConnectionParams;
import org.apache.commons.httpclient.protocol.ProtocolSocketFactory;
import org.apache.commons.httpclient.protocol.SecureProtocolSocketFactory;

import eu.unicore.gateway.properties.SecurityProperties;
import eu.unicore.gateway.util.TruststoreUtil;

/**
 * Code from Commons HTTPClient "contrib" section <br/>
 * 
 *  
 * <p>
 * AuthSSLProtocolSocketFactory can be used to validate the identity of the HTTPS 
 * server against a list of trusted certificates and to authenticate to the HTTPS 
 * server using a private key. 
 * </p>
 * 
 * <p>
 * AuthSSLProtocolSocketFactory will enable server authentication when supplied with
 * a {@link KeyStore truststore} file containg one or several trusted certificates. 
 * The client secure socket will reject the connection during the SSL session handshake 
 * if the target HTTPS server attempts to authenticate itself with a non-trusted 
 * certificate.
 * </p>
 * 
 * @author <a href="mailto:oleg -at- ural.ru">Oleg Kalnichevski</a>
 * 
 * <p>
 * DISCLAIMER: HttpClient developers DO NOT actively support this component.
 * The component is provided as a reference material, which may be inappropriate
 * for use without additional customization.
 * </p>
 */

public class AuthSSLProtocolSocketFactory implements ProtocolSocketFactory {

	private static final Logger LOG =Logger.getLogger("unicore.gateway."+AuthSSLProtocolSocketFactory.class.getSimpleName()); 

	private SSLContext sslcontext = null;

	private SecurityProperties sec;

	/**
	 * Constructor for AuthSSLProtocolSocketFactory. Either a keystore or truststore file
	 * must be given. Otherwise SSL context initialization error will result.
	 * 
	 * @param keystoreUrl URL of the keystore file. May be <tt>null</tt> if HTTPS client
	 *        authentication is not to be used.
	 * @param keystorePassword Password to unlock the keystore. IMPORTANT: this implementation
	 *        assumes that the same password is used to protect the key and the keystore itself.
	 * @param truststoreUrl URL of the truststore file. May be <tt>null</tt> if HTTPS server
	 *        authentication is not to be used.
	 * @param truststorePassword Password to unlock the truststore.
	 */
	public AuthSSLProtocolSocketFactory(final SecurityProperties sec)
	{
		super();
		this.sec=sec;
	}

	private static KeyStore createKeyStore(String name, String passwd, String type, String alias) 
	throws KeyStoreException, NoSuchAlgorithmException, CertificateException, IOException
	{
		if (name == null) {
			throw new IllegalArgumentException("Keystore/Truststore name may not be null");
		}
		String storeType=type!=null ? type : "jks";
		InputStream is = null;
		KeyStore keystore = null;
		if("file".equalsIgnoreCase(type)){
			return TruststoreUtil.loadTruststoreFromPemFile(name);
		}
		else if(type.startsWith("dir")){
			return TruststoreUtil.loadTruststoreFromDirectory(name);
		}
		else{
			try {
				is = new FileInputStream(name); 
				keystore  = KeyStore.getInstance(storeType);
				keystore.load(is, passwd!= null ? passwd.toCharArray(): null);
			} finally {
				if (is != null) is.close();
			}
		}

		//if we have an alias, delete all other keys from this keystore to make
		//sure the SSL connection uses the right one
		if(alias!=null && alias.trim().length()!=0){
			LOG.debug("Using alias <"+alias+">");
			Enumeration<String> en=keystore.aliases();
			List<String> delete=new ArrayList<String>();
			while(en.hasMoreElements()){
				String a=en.nextElement();
				if(!alias.equalsIgnoreCase(a) && keystore.isKeyEntry(a)){
					LOG.debug("Deleting non-needed key "+a);
					delete.add(a);
				}
			}
			for(String del: delete)keystore.deleteEntry(del);
		}

		return keystore;
	}

	private static KeyManager[] createKeyManagers(final KeyStore keystore, final String password)
	throws KeyStoreException, NoSuchAlgorithmException, UnrecoverableKeyException 
	{
		if (keystore == null) {
			throw new IllegalArgumentException("Keystore may not be null");
		}
		KeyManagerFactory kmfactory = KeyManagerFactory.getInstance(
				KeyManagerFactory.getDefaultAlgorithm());
		kmfactory.init(keystore, password != null ? password.toCharArray(): null);
		return kmfactory.getKeyManagers(); 
	}

	private static TrustManager[] createTrustManagers(final KeyStore keystore)
	throws KeyStoreException, NoSuchAlgorithmException
	{ 
		if (keystore == null) {
			throw new IllegalArgumentException("Keystore may not be null");
		}
		TrustManagerFactory tmfactory = TrustManagerFactory.getInstance(
				TrustManagerFactory.getDefaultAlgorithm());
		tmfactory.init(keystore);
		TrustManager[] trustmanagers = tmfactory.getTrustManagers();
		return trustmanagers; 
	}

	private synchronized SSLContext createSSLContext() {
		if(sslcontext!=null)return sslcontext;
		try {
			KeyManager[] keymanagers = null;
			TrustManager[] trustmanagers = null;
			if (sec.getKeystore()!= null) {
				KeyStore keystore = createKeyStore(sec.getKeystore(),sec.getKeystorePassword(),sec.getKeystoreType(),sec.getKeystoreAlias());
				if (LOG.isDebugEnabled()) {
					Enumeration<String> aliases = keystore.aliases();
					while (aliases.hasMoreElements()) {
						String alias = (String)aliases.nextElement();                        
						Certificate[] certs = keystore.getCertificateChain(alias);
						if (certs != null) {
							LOG.debug("Certificate chain '" + alias + "':");
							for (int c = 0; c < certs.length; c++) {
								if (certs[c] instanceof X509Certificate) {
									X509Certificate cert = (X509Certificate)certs[c];
									LOG.debug(" Certificate " + (c + 1) + ":");
									LOG.debug("  Subject DN: " + cert.getSubjectDN());
									LOG.debug("  Signature Algorithm: " + cert.getSigAlgName());
									LOG.debug("  Valid from: " + cert.getNotBefore() );
									LOG.debug("  Valid until: " + cert.getNotAfter());
									LOG.debug("  Issuer: " + cert.getIssuerDN());
								}
							}
						}
					}
				}
				keymanagers = createKeyManagers(keystore, sec.getKeystorePassword());
			}
			if (sec.getTruststore() != null) {
				KeyStore truststore = createKeyStore(sec.getTruststore(),sec.getTruststorePassword(),sec.getTruststoreType(),null);
				if (LOG.isDebugEnabled()) {
					Enumeration<String> aliases = truststore.aliases();
					while (aliases.hasMoreElements()) {
						String alias = (String)aliases.nextElement();
						LOG.debug("Trusted certificate '" + alias + "':");
						Certificate trustedcert = truststore.getCertificate(alias);
						if (trustedcert != null && trustedcert instanceof X509Certificate) {
							X509Certificate cert = (X509Certificate)trustedcert;
							LOG.debug("  Subject DN: " + cert.getSubjectDN());
							LOG.debug("  Signature Algorithm: " + cert.getSigAlgName());
							LOG.debug("  Valid from: " + cert.getNotBefore() );
							LOG.debug("  Valid until: " + cert.getNotAfter());
							LOG.debug("  Issuer: " + cert.getIssuerDN());
						}
					}
				}
				trustmanagers = createTrustManagers(truststore);
			}
			SSLContext sslcontext = SSLContext.getInstance("SSL");
			sslcontext.init(keymanagers, trustmanagers, null);
			return sslcontext;
		} catch (Exception e) {
			LOG.fatal(e.getMessage(), e);
			throw new RuntimeException(e);
		}
	}

	private SSLContext getSSLContext() {
		if (this.sslcontext == null) {
			this.sslcontext = createSSLContext();
		}
		return this.sslcontext;
	}

	/**
	 * Attempts to get a new socket connection to the given host within the given time limit.
	 * <p>
	 * To circumvent the limitations of older JREs that do not support connect timeout a 
	 * controller thread is executed. The controller thread attempts to create a new socket 
	 * within the given limit of time. If socket constructor does not return until the 
	 * timeout expires, the controller terminates and throws an {@link ConnectTimeoutException}
	 * </p>
	 *  
	 * @param host the host name/IP
	 * @param port the port on the host
	 * @param clientHost the local host name/IP to bind the socket to
	 * @param clientPort the port on the local machine
	 * @param params {@link HttpConnectionParams Http connection parameters}
	 * 
	 * @return Socket a new socket
	 * 
	 * @throws IOException if an I/O error occurs while creating the socket
	 * @throws UnknownHostException if the IP address of the host cannot be
	 * determined
	 */
	public Socket createSocket(
			final String host,
			final int port,
			final InetAddress localAddress,
			final int localPort,
			final HttpConnectionParams params
	) throws IOException, UnknownHostException, ConnectTimeoutException {
		if (params == null) {
			throw new IllegalArgumentException("Parameters may not be null");
		}
		int timeout = params.getConnectionTimeout();
		SocketFactory socketfactory = getSSLContext().getSocketFactory();
		if (timeout == 0) {
			return socketfactory.createSocket(host, port, localAddress, localPort);
		} else {
			Socket socket = socketfactory.createSocket();
			SocketAddress localaddr = new InetSocketAddress(localAddress, localPort);
			SocketAddress remoteaddr = new InetSocketAddress(host, port);
			socket.bind(localaddr);
			socket.connect(remoteaddr, timeout);
			return socket;
		}
	}

	/**
	 * @see SecureProtocolSocketFactory#createSocket(java.lang.String,int,java.net.InetAddress,int)
	 */
	public Socket createSocket(
			String host,
			int port,
			InetAddress clientHost,
			int clientPort)
	throws IOException, UnknownHostException
	{
		return getSSLContext().getSocketFactory().createSocket(
				host,
				port,
				clientHost,
				clientPort
		);
	}

	/**
	 * @see SecureProtocolSocketFactory#createSocket(java.lang.String,int)
	 */
	public Socket createSocket(String host, int port)
	throws IOException, UnknownHostException
	{
		return getSSLContext().getSocketFactory().createSocket(
				host,
				port
		);
	}

	/**
	 * @see SecureProtocolSocketFactory#createSocket(java.net.Socket,java.lang.String,int,boolean)
	 */
	public Socket createSocket(
			Socket socket,
			String host,
			int port,
			boolean autoClose)
	throws IOException, UnknownHostException
	{
		return getSSLContext().getSocketFactory().createSocket(
				socket,
				host,
				port,
				autoClose
		);
	}
}

==> unicore-gateway6-4.3.0-2.src.tar.gz: src/main/java/eu/unicore/gateway/client/AuthSSLProtocolSocketFactory.java <==
package eu.unicore.gateway.client;

import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.SocketAddress;
import java.net.UnknownHostException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import javax.net.SocketFactory;
import javax.net.ssl.KeyManager;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.TrustManagerFactory;

import org.apache.commons.httpclient.ConnectTimeoutException;
import org.apache.commons.httpclient.params.HttpConnectionParams;
import org.apache.commons.httpclient.protocol.ProtocolSocketFactory;
import org.apache.commons.httpclient.protocol.SecureProtocolSocketFactory;
import org.apache.log4j.Logger;

import eu.unicore.gateway.properties.SecurityProperties;
import eu.unicore.security.util.KeystoreUtil;

/**
 * Code from Commons HTTPClient "contrib" section <br/>
 * 
 *  
 * <p>
 * AuthSSLProtocolSocketFactory can be used to validate the identity of the HTTPS 
 * server against a list of trusted certificates and to authenticate to the HTTPS 
 * server using a private key. 
 * </p>
 * 
 * <p>
 * AuthSSLProtocolSocketFactory will enable server authentication when supplied with
 * a {@link KeyStore truststore} file containg one or several trusted certificates. 
 * The client secure socket will reject the connection during the SSL session handshake 
 * if the target HTTPS server attempts to authenticate itself with a non-trusted 
 * certificate.
 * </p>
 * 
 * @author <a href="mailto:oleg -at- ural.ru">Oleg Kalnichevski</a>
 * 
 * <p>
 * DISCLAIMER: HttpClient developers DO NOT actively support this component.
 * The component is provided as a reference material, which may be inappropriate
 * for use without additional customization.
 * </p>
 * 
 * TODO: we could probably use the equivalent class {@link eu.unicore.security.util.client.AuthSSLProtocolSocketFactory}
 */
public class AuthSSLProtocolSocketFactory implements ProtocolSocketFactory {

	private static final Logger LOG =Logger.getLogger("unicore.gateway."+AuthSSLProtocolSocketFactory.class.getSimpleName()); 

	private SSLContext sslcontext = null;

	private SecurityProperties sec;

	/**
	 * Constructor for AuthSSLProtocolSocketFactory. Either a keystore or truststore file
	 * must be given. Otherwise SSL context initialization error will result.
	 * 
	 * @param keystoreUrl URL of the keystore file. May be <tt>null</tt> if HTTPS client
	 *        authentication is not to be used.
	 * @param keystorePassword Password to unlock the keystore. IMPORTANT: this implementation
	 *        assumes that the same password is used to protect the key and the keystore itself.
	 * @param truststoreUrl URL of the truststore file. May be <tt>null</tt> if HTTPS server
	 *        authentication is not to be used.
	 * @param truststorePassword Password to unlock the truststore.
	 */
	public AuthSSLProtocolSocketFactory(final SecurityProperties sec)
	{
		super();
		this.sec=sec;
	}

	private static KeyStore createKeyStore(String name, String passwd, String type, String alias) 
	throws KeyStoreException, NoSuchAlgorithmException, CertificateException, IOException
	{
		if (name == null) {
			throw new IllegalArgumentException("Keystore/Truststore name may not be null");
		}
		
		KeyStore keystore = KeystoreUtil.loadKeyStore(name, passwd, type);
		
		//if we have an alias, delete all other keys from this keystore to make
		//sure the SSL connection uses the right one
		if(alias!=null && alias.trim().length()!=0){
			LOG.debug("Using alias <"+alias+">");
			Enumeration<String> en=keystore.aliases();
			List<String> delete=new ArrayList<String>();
			while(en.hasMoreElements()){
				String a=en.nextElement();
				if(!alias.equalsIgnoreCase(a) && keystore.isKeyEntry(a)){
					LOG.debug("Deleting non-needed key "+a);
					delete.add(a);
				}
			}
			for(String del: delete)keystore.deleteEntry(del);
		}

		return keystore;
	}

	private static KeyManager[] createKeyManagers(final KeyStore keystore, final String password)
	throws KeyStoreException, NoSuchAlgorithmException, UnrecoverableKeyException 
	{
		if (keystore == null) {
			throw new IllegalArgumentException("Keystore may not be null");
		}
		KeyManagerFactory kmfactory = KeyManagerFactory.getInstance(
				KeyManagerFactory.getDefaultAlgorithm());
		kmfactory.init(keystore, password != null ? password.toCharArray(): null);
		return kmfactory.getKeyManagers(); 
	}

	private static TrustManager[] createTrustManagers(final KeyStore keystore)
	throws KeyStoreException, NoSuchAlgorithmException
	{ 
		if (keystore == null) {
			throw new IllegalArgumentException("Keystore may not be null");
		}
		TrustManagerFactory tmfactory = TrustManagerFactory.getInstance(
				TrustManagerFactory.getDefaultAlgorithm());
		tmfactory.init(keystore);
		TrustManager[] trustmanagers = tmfactory.getTrustManagers();
		return trustmanagers; 
	}

	private synchronized SSLContext createSSLContext() {
		if(sslcontext!=null)return sslcontext;
		try {
			KeyManager[] keymanagers = null;
			TrustManager[] trustmanagers = null;
			if (sec.getKeystore()!= null) {
				KeyStore keystore = createKeyStore(sec.getKeystore(),sec.getKeystorePassword(),sec.getKeystoreType(),sec.getKeystoreAlias());
				if (LOG.isDebugEnabled()) {
					Enumeration<String> aliases = keystore.aliases();
					while (aliases.hasMoreElements()) {
						String alias = (String)aliases.nextElement();                        
						Certificate[] certs = keystore.getCertificateChain(alias);
						if (certs != null) {
							LOG.debug("Certificate chain '" + alias + "':");
							for (int c = 0; c < certs.length; c++) {
								if (certs[c] instanceof X509Certificate) {
									X509Certificate cert = (X509Certificate)certs[c];
									LOG.debug(" Certificate " + (c + 1) + ":");
									LOG.debug("  Subject DN: " + cert.getSubjectDN());
									LOG.debug("  Signature Algorithm: " + cert.getSigAlgName());
									LOG.debug("  Valid from: " + cert.getNotBefore() );
									LOG.debug("  Valid until: " + cert.getNotAfter());
									LOG.debug("  Issuer: " + cert.getIssuerDN());
								}
							}
						}
					}
				}
				keymanagers = createKeyManagers(keystore, sec.getKeystorePassword());
			}
			if (sec.getTruststore() != null) {
				KeyStore truststore = KeystoreUtil.loadTruststore(sec.getTruststore(),sec.getTruststorePassword(),sec.getTruststoreType());
				if (LOG.isDebugEnabled()) {
					Enumeration<String> aliases = truststore.aliases();
					while (aliases.hasMoreElements()) {
						String alias = (String)aliases.nextElement();
						LOG.debug("Trusted certificate '" + alias + "':");
						Certificate trustedcert = truststore.getCertificate(alias);
						if (trustedcert != null && trustedcert instanceof X509Certificate) {
							X509Certificate cert = (X509Certificate)trustedcert;
							LOG.debug("  Subject DN: " + cert.getSubjectDN());
							LOG.debug("  Signature Algorithm: " + cert.getSigAlgName());
							LOG.debug("  Valid from: " + cert.getNotBefore() );
							LOG.debug("  Valid until: " + cert.getNotAfter());
							LOG.debug("  Issuer: " + cert.getIssuerDN());
						}
					}
				}
				trustmanagers = createTrustManagers(truststore);
			}
			SSLContext sslcontext = SSLContext.getInstance("SSL");
			sslcontext.init(keymanagers, trustmanagers, null);
			return sslcontext;
		} catch (Exception e) {
			LOG.fatal(e.getMessage(), e);
			throw new RuntimeException(e);
		}
	}

	private SSLContext getSSLContext() {
		if (this.sslcontext == null) {
			this.sslcontext = createSSLContext();
		}
		return this.sslcontext;
	}

	/**
	 * Attempts to get a new socket connection to the given host within the given time limit.
	 * <p>
	 * To circumvent the limitations of older JREs that do not support connect timeout a 
	 * controller thread is executed. The controller thread attempts to create a new socket 
	 * within the given limit of time. If socket constructor does not return until the 
	 * timeout expires, the controller terminates and throws an {@link ConnectTimeoutException}
	 * </p>
	 *  
	 * @param host the host name/IP
	 * @param port the port on the host
	 * @param clientHost the local host name/IP to bind the socket to
	 * @param clientPort the port on the local machine
	 * @param params {@link HttpConnectionParams Http connection parameters}
	 * 
	 * @return Socket a new socket
	 * 
	 * @throws IOException if an I/O error occurs while creating the socket
	 * @throws UnknownHostException if the IP address of the host cannot be
	 * determined
	 */
	public Socket createSocket(
			final String host,
			final int port,
			final InetAddress localAddress,
			final int localPort,
			final HttpConnectionParams params
	) throws IOException, UnknownHostException, ConnectTimeoutException {
		if (params == null) {
			throw new IllegalArgumentException("Parameters may not be null");
		}
		int timeout = params.getConnectionTimeout();
		SocketFactory socketfactory = getSSLContext().getSocketFactory();
		if (timeout == 0) {
			return socketfactory.createSocket(host, port, localAddress, localPort);
		} else {
			Socket socket = socketfactory.createSocket();
			SocketAddress localaddr = new InetSocketAddress(localAddress, localPort);
			SocketAddress remoteaddr = new InetSocketAddress(host, port);
			socket.bind(localaddr);
			socket.connect(remoteaddr, timeout);
			return socket;
		}
	}

	/**
	 * @see SecureProtocolSocketFactory#createSocket(java.lang.String,int,java.net.InetAddress,int)
	 */
	public Socket createSocket(
			String host,
			int port,
			InetAddress clientHost,
			int clientPort)
	throws IOException, UnknownHostException
	{
		return getSSLContext().getSocketFactory().createSocket(
				host,
				port,
				clientHost,
				clientPort
		);
	}

	/**
	 * @see SecureProtocolSocketFactory#createSocket(java.lang.String,int)
	 */
	public Socket createSocket(String host, int port)
	throws IOException, UnknownHostException
	{
		return getSSLContext().getSocketFactory().createSocket(
				host,
				port
		);
	}

	/**
	 * @see SecureProtocolSocketFactory#createSocket(java.net.Socket,java.lang.String,int,boolean)
	 */
	public Socket createSocket(
			Socket socket,
			String host,
			int port,
			boolean autoClose)
	throws IOException, UnknownHostException
	{
		return getSSLContext().getSocketFactory().createSocket(
				socket,
				host,
				port,
				autoClose
		);
	}
}
